package yukihane.inqubus.conifg;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.XMLConfiguration;

public enum Properties implements Config {

    INSTANCE;
    private final XMLConfiguration config;
    /** ニコニコ動画ログインID */
    private String p_nicovideo_id;
    /** ニコニコ動画ログインパスワード */
    private String p_nicovideo_password;
    /** プロキシを使用する */
    private String p_proxy_use;
    /** プロキシホスト */
    private String p_proxy_host;
    /** プロキシポート */
    private String p_proxy_port;
    /** NicoBrowser情報を参照する */
    private String p_nicobrowser_use;
    /** NicoBrowser DB ディレクトリ */
    private String p_nicobrowser_dblocation;
    /** 動画をダウンロードする */
    private String p_movie_download;
    /** 動画ダウンロードディレクトリ */
    private String p_movie_directory;
    /** コメントをダウンロードする */
    private String p_comment_download;
    /** コメントダウンロードディレクトリ */
    private String p_comment_directory;
    /** 投稿者コメントをダウンロードする */
    private String p_ownercomment_download;
    /** 投稿者コメントダウンロードディレクトリ */
    private String p_ownercomment_directory;

    @Override
    public boolean isOwnerCommentDownload() {
        return Boolean.valueOf(p_ownercomment_download);
    }

    public void setOwnerCommentDownload(boolean download) {
        p_ownercomment_download = Boolean.toString(download);
    }

    @Override
    public boolean isCommentDownload() {
        return Boolean.valueOf(p_comment_download);
    }

    public void setCommentDownload(boolean download) {
        p_comment_download = Boolean.toString(download);
    }

    @Override
    public String getMovieDirectory() {
        return p_movie_directory;
    }

    public void setMovieDirectory(String dir) {
        this.p_movie_directory = dir;
    }

    @Override
    public boolean isMovieDownload() {
        return Boolean.valueOf(p_movie_download);
    }

    public void setMovieDownload(boolean download) {
        this.p_movie_download = Boolean.toString(download);
    }

    @Override
    public String getDbLocation() {
        return p_nicobrowser_dblocation;
    }

    public void setDbLocation(String p_nicobrowser_dblocation) {
        this.p_nicobrowser_dblocation = p_nicobrowser_dblocation;
    }

    @Override
    public boolean isDbUse() {
        return Boolean.valueOf(p_nicobrowser_use);
    }

    public void setDbUse(boolean use) {
        this.p_nicobrowser_use = Boolean.toString(use);
    }

    @Override
    public String getNicoVideoId() {
        return p_nicovideo_id;
    }

    public void setNicoVideoId(String id) {
        this.p_nicovideo_id = id;
    }

    @Override
    public String getNicoVideoPassword() {
        return p_nicovideo_password;
    }

    public void setNicoVideoPassword(String password) {
        this.p_nicovideo_password = password;
    }

    @Override
    public String getProxyHost() {
        return p_proxy_host;
    }

    public void setProxyHost(String host) {
        this.p_proxy_host = host;
    }

    @Override
    public int getP_proxy_port() {
        return Integer.parseInt(p_proxy_port);
    }

    public void setP_proxy_port(int port) {
        this.p_proxy_port = Integer.toString(port);
    }

    @Override
    public boolean isProxyUse() {
        return Boolean.valueOf(p_proxy_use);
    }

    public void setProxyUse(boolean use) {
        this.p_proxy_use = Boolean.toString(use);
    }

    private String getConfigurationName() {
        return "inqubus.xml";
    }

    private String getPrefix() {
        return "p_";
    }

    private Properties() {
        config = new XMLConfiguration();
        config.setFileName(getConfigurationName());
    }

    public void load() throws ConfigurationException {
        config.load();
    }

    public void save() throws ConfigurationException {
        try {
            final Class<? extends Properties> clazz = this.getClass();
            final Field[] fields = clazz.getDeclaredFields();
            final Field[] propertyFields = getPropertyFields(fields);
            for (Field f : propertyFields) {
                config.setProperty(getPropertyName(f), getPropertyValue(f));
            }
            config.save();
        } catch (Exception ex) {
            throw new ConfigurationException(ex);
        }
    }

    private Field[] getPropertyFields(Field[] fields) {
        List<Field> res = new ArrayList<Field>();
        for (Field f : fields) {
            if (f.getName().startsWith(getPrefix())) {
                res.add(f);
            }
        }
        return res.toArray(new Field[0]);
    }

    private String getPropertyName(Field field) {
        return field.getName().replace(getPrefix(), "").replace("_", ".");
    }

    private String getPropertyValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        field.setAccessible(true);
        return (String) field.get(this);
    }
}
